@@ -4,6 +4,8 @@ import warnings
 import sys
 
 import pymysql
+from pymysql import cursors
+from pymysql._compat import text_type
 from pymysql.tests import base
 import unittest2
 
@@ -34,7 +36,7 @@ class TestOldIssues(base.PyMySQLTestCase):
             c.execute("select dt from issue3")
             self.assertEqual(None, c.fetchone()[0])
             c.execute("select ts from issue3")
-            self.assertTrue(isinstance(c.fetchone()[0], datetime.datetime))
+            self.assertIn(type(c.fetchone()[0]), (type(None), datetime.datetime), 'expected Python type None or datetime from SQL timestamp')
         finally:
             c.execute("drop table issue3")
 
@@ -145,7 +147,7 @@ KEY (`station`,`dh`,`echeance`)) ENGINE=MyISAM DEFAULT CHARSET=latin1;""")
 
     @unittest2.skip("test_issue_17() requires a custom, legacy MySQL configuration and will not be run.")
     def test_issue_17(self):
-        """ could not connect mysql use passwod """
+        """could not connect mysql use passwod"""
         conn = self.connections[0]
         host = self.databases[0]["host"]
         db = self.databases[0]["db"]
@@ -407,15 +409,15 @@ class TestGitHubIssues(base.PyMySQLTestCase):
 
     def test_issue_364(self):
         """ Test mixed unicode/binary arguments in executemany. """
-        conn = pymysql.connect(charset="utf8", **self.databases[0])
+        conn = pymysql.connect(charset="utf8mb4", **self.databases[0])
         self.safe_create_table(
             conn, "issue364",
             "create table issue364 (value_1 binary(3), value_2 varchar(3)) "
-            "engine=InnoDB default charset=utf8")
+            "engine=InnoDB default charset=utf8mb4")
 
-        sql = "insert into issue364 (value_1, value_2) values (_binary%s, _binary%s)"
-        usql = u"insert into issue364 (value_1, value_2) values (_binary%s, _binary%s)"
-        values = [b"\x00\xff\x00", u"\xe4\xf6\xfc"]
+        sql = "insert into issue364 (value_1, value_2) values (_binary %s, %s)"
+        usql = u"insert into issue364 (value_1, value_2) values (_binary %s, %s)"
+        values = [pymysql.Binary(b"\x00\xff\x00"), u"\xe4\xf6\xfc"]
 
         # test single insert and select
         cur = conn.cursor()
@@ -441,35 +443,33 @@ class TestGitHubIssues(base.PyMySQLTestCase):
         self.safe_create_table(
             conn, "issue363",
             "CREATE TABLE issue363 ( "
-            "id INTEGER PRIMARY KEY, geom LINESTRING NOT NULL, "
+            "id INTEGER PRIMARY KEY, geom LINESTRING NOT NULL /*!80003 SRID 0 */, "
             "SPATIAL KEY geom (geom)) "
-            "ENGINE=MyISAM default charset=utf8")
+            "ENGINE=MyISAM")
 
         cur = conn.cursor()
-        # FYI - not sure of 5.7.0 version
-        if sys.version_info[0:2] >= (3,2) and self.mysql_server_is(conn, (5, 7, 0)):
-            with self.assertWarns(pymysql.err.Warning) as cm:
-                cur.execute("INSERT INTO issue363 (id, geom) VALUES ("
-                            "1998, GeomFromText('LINESTRING(1.1 1.1,2.2 2.2)'))")
+        # From MySQL 5.7, ST_GeomFromText is added and GeomFromText is deprecated.
+        if self.mysql_server_is(conn, (5, 7, 0)):
+            geom_from_text = "ST_GeomFromText"
+            geom_as_text = "ST_AsText"
+            geom_as_bin = "ST_AsBinary"
         else:
-            cur.execute("INSERT INTO issue363 (id, geom) VALUES ("
-                        "1998, GeomFromText('LINESTRING(1.1 1.1,2.2 2.2)'))")
+            geom_from_text = "GeomFromText"
+            geom_as_text = "AsText"
+            geom_as_bin = "AsBinary"
+        query = ("INSERT INTO issue363 (id, geom) VALUES"
+                 "(1998, %s('LINESTRING(1.1 1.1,2.2 2.2)'))" % geom_from_text)
+        cur.execute(query)
 
         # select WKT
-        if sys.version_info[0:2] >= (3,2) and self.mysql_server_is(conn, (5, 7, 0)):
-            with self.assertWarns(pymysql.err.Warning) as cm:
-                cur.execute("SELECT AsText(geom) FROM issue363")
-        else:
-            cur.execute("SELECT AsText(geom) FROM issue363")
+        query = "SELECT %s(geom) FROM issue363" % geom_as_text
+        cur.execute(query)
         row = cur.fetchone()
         self.assertEqual(row, ("LINESTRING(1.1 1.1,2.2 2.2)", ))
 
         # select WKB
-        if sys.version_info[0:2] >= (3,2) and self.mysql_server_is(conn, (5, 7, 0)):
-            with self.assertWarns(pymysql.err.Warning) as cm:
-                cur.execute("SELECT AsBinary(geom) FROM issue363")
-        else:
-            cur.execute("SELECT AsBinary(geom) FROM issue363")
+        query = "SELECT %s(geom) FROM issue363" % geom_as_bin
+        cur.execute(query)
         row = cur.fetchone()
         self.assertEqual(row,
                          (b"\x01\x02\x00\x00\x00\x02\x00\x00\x00"
@@ -484,3 +484,28 @@ class TestGitHubIssues(base.PyMySQLTestCase):
         # don't assert the exact internal binary value, as it could
         # vary across implementations
         self.assertTrue(isinstance(row[0], bytes))
+
+    def test_issue_491(self):
+        """ Test warning propagation """
+        conn = pymysql.connect(charset="utf8", **self.databases[0])
+
+        with warnings.catch_warnings():
+            # Ignore all warnings other than pymysql generated ones
+            warnings.simplefilter("ignore")
+            warnings.simplefilter("error", category=pymysql.Warning)
+
+            # verify for both buffered and unbuffered cursor types
+            for cursor_class in (cursors.Cursor, cursors.SSCursor):
+                c = conn.cursor(cursor_class)
+                try:
+                    c.execute("SELECT CAST('124b' AS SIGNED)")
+                    c.fetchall()
+                except pymysql.Warning as e:
+                    # Warnings should have errorcode and string message, just like exceptions
+                    self.assertEqual(len(e.args), 2)
+                    self.assertEqual(e.args[0], 1292)
+                    self.assertTrue(isinstance(e.args[1], text_type))
+                else:
+                    self.fail("Should raise Warning")
+                finally:
+                    c.close()
